/////////////////////////////////////////////////////////////////
// MakeCoords.h
/////////////////////////////////////////////////////////////////

#include <iostream>
#include <vector>
#include <string>
#include <sstream>
#include <fstream>
#include <cstring>
#include <cstdio>
#include "Minimizer.h"
#include "Utilities.h"
#include "SStruct.h"

using namespace std;

/////////////////////////////////////////////////////////////////
// Constants
/////////////////////////////////////////////////////////////////

//const int MAX_STEM_DIAGONAL_CONSTRAINT = 3;
//const double DIAG_STRENGTH = 20;
const double LOOP_STRENGTH = 30;
//const double SUBTREE_REPULSIVE_STRENGTH = 5;
const double REPULSIVE_STRENGTH = 0.2;
const double BACKBONE_STRENGTH = 50;
const double PI = 3.141592653589793238462643383279502884197169399;
const int STEM_WIDTH = 3;

/////////////////////////////////////////////////////////////////
// Types
/////////////////////////////////////////////////////////////////

struct Point {
  double x, y;
  Point () : x(0), y(0) {}
  Point (double x, double y) : x(x), y(y) {}
};

enum ConstraintType {
  ConstraintType_LENGTH,
  ConstraintType_REPULSIVE
};

struct Constraint {
  ConstraintType type;
  int i;
  int j;
  double dist;
  double strength;
  
  Constraint (ConstraintType type, int i, int j, double dist, double strength) : 
    type(type), i(i), j(j), dist(dist), strength(strength) {}
};

typedef vector<int> VI;
typedef vector<VI> VVI;
typedef vector<double> VD;
typedef vector<string> VS;

/////////////////////////////////////////////////////////////////
// class MakeCoords
/////////////////////////////////////////////////////////////////

class MakeCoords : public Minimizer {
  const SStruct &sstruct;
  vector<Constraint> constraints;

 public:

  vector<Point> coords;
  vector<Point> gradients;
  
  VD GetParams (const vector<Point> &points) const;
  void SetParams (const VD &values);
  void PrintParams (const string &filename) const;

  VI ComputeLoop (const VI &mapping, int left) const;
  Point ComputeLoopCenter (Point p1, Point p2, int k, int n) const;
  vector<Point> ComputeLoopPositions (Point p1, Point center, int n) const;
  void InitialPlacement();

  double Distance (Point p, Point q) const;
  void AddConstraints();


  MakeCoords (const SStruct &sstruct);
  void ComputeGradient (vector<double> &gradient, const VD &values);
  double ComputeFunction (const VD &values);
  void Report (const vector<double> &theta, int iteration, 
	       double objective, double step_length);
  void Report (const string &s);

  double CrossProduct (const Point &a, Point b, Point c) const;
  bool IsCrossing (const Point &p1, const Point &p2, const Point &q1, const Point &q2) const;
  VI IdentifyCrossingResidues();

};
