/////////////////////////////////////////////////////////////////
// SStruct.h
/////////////////////////////////////////////////////////////////

#ifndef SSTRUCT_H
#define SSTRUCT_H

#include <fstream>
#include <vector>
#include <string>
#include <sstream>
#include "Utilities.h"

using namespace std;

const int UNKNOWN = -1;
const int UNPAIRED = 0;

/////////////////////////////////////////////////////////////////
// enum FileFormat
/////////////////////////////////////////////////////////////////

enum FileFormat { 
  FileFormat_FASTA,
  FileFormat_BPSEQ,
  FileFormat_RAW
};                  

/////////////////////////////////////////////////////////////////
// class SStruct
/////////////////////////////////////////////////////////////////

class SStruct {
  string sequence;
  vector<int> mapping;
  string name;

  void AssertValidSequence (const string &sequence) const;
  void AssertValidMapping (const vector<int> &mapping) const;

 public:
  
  SStruct (const string &filename);
  SStruct (const string &sequence, const vector<int> &mapping, const string &name);
  SStruct (const SStruct &rhs);
  const SStruct& operator= (const SStruct &rhs);

  const string &GetSequence () const;
  const vector<int> &GetMapping () const;
  const string &GetName () const;
  int GetLength () const;

  void LoadFASTA (const string &filename);
  void LoadRAW (const string &filename);
  void LoadBPSEQ (const string &filename);
  
  void WriteBPSEQ (ostream &outfile) const;
  void WriteParens (ostream &outfile) const;

  static bool IsComplementary (char c, char d);
  vector<int> GetComplementaryMapping () const;

};

#endif
