/////////////////////////////////////////////////////////////////
// Utilities.cc
/////////////////////////////////////////////////////////////////

#include "Utilities.h"
#include <stdlib.h>

bool toggle_error = false;

/////////////////////////////////////////////////////////////////
// _ASSERT_FAILED()
// 
// Print error message for a failed assertion and terminate.
/////////////////////////////////////////////////////////////////

int _ASSERT_FAILED (const char *fmt, ...){
  if (toggle_error) return 0;
  va_list ap;
  va_start(ap, fmt);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
  abort();
  return 0;
}

/////////////////////////////////////////////////////////////////
// Error()
// 
// Print error message for a user error and terminate.
/////////////////////////////////////////////////////////////////

void Error (const char *fmt, ...){
  fprintf (stderr, "ERROR: ");
  va_list ap;
  va_start(ap, fmt);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
  fprintf (stderr, "\n");
  toggle_error = true;
  exit (1);
}

/////////////////////////////////////////////////////////////////
// ConvertToDouble()
//
// Attempts to parse a float from the character string given.
// Returns true only if no parsing error occurs.
/////////////////////////////////////////////////////////////////
 
bool ConvertToDouble (const string &s, double &val){
  char *end_ptr;
  errno = 0;
  val = strtod (s.c_str(), &end_ptr);
  if (val == 0 && (errno != 0 || s.c_str() == end_ptr)) return false;
  if (errno != 0 && (val >= 1000000.0 || val <= -1000000.0)) return false;
  return true;
}

/////////////////////////////////////////////////////////////////
// SPrintF()
//
// Same as sprintf but returns a string.
/////////////////////////////////////////////////////////////////

string SPrintF (const char *fmt, ...){

  int buf_size = 1024;
  char *buffer = new char[buf_size];
  Assert (buffer, "Failed to allocate memory.");
  
  while (true){

    // print text to buffer

    va_list ap;
    va_start (ap, fmt);
    int num_written = vsnprintf(buffer, buf_size, fmt, ap);
    va_end(ap);

    // double size of buffer if needed
    
    if (num_written >= buf_size){
      char *temp = new char[buf_size*2];
      Assert (temp, "Failed to allocate memory.");
      memcpy (temp, buffer, sizeof(char) * buf_size);
      delete [] buffer;
      buffer = temp;
      buf_size *= 2;      
    } else {
      break;
    }
  }

  // return text

  const string s (buffer);
  delete [] buffer;
  return s;
}
